//..............................................................................
// -*- Mode: Java; -*-
//                          Package   : DataBrowser
// BasicFrame.java          Author    : Deanna Evans (dke),
//                                      Ed Kraemer (eck) and,
//                                      Mark Pumar (map)
//
//  This file is part of DataBrowser.
//
//  DataBrowser is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
// Description: Window (frame) that serves as a base class for creating
//              Java application windows.  The BasicFrame class includes
//              standard window frame icons for closing an application window
//              and constucting menus.
//..............................................................................
//
package BasicFrame;
// Imports
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

//..............................................................................
//
// Description:
//
public class BasicFrame extends JFrame
{
   //...........................................................................
   //
   // The BasicFrameMouseMoved captures mouse moved events.  This method
   // does nothing in the BasicFrame class and can be overloaded in any
   // child class.
   //
   public void BasicFrameMouseMoved(MouseEvent e)
   {};
   //...........................................................................
   //
   // Description: The BasicFrame method is the class constructor. It caputes
   //              mouse moved events and passes them along, establishes the
   //              default window size, and caputes window close events and
   //              causes the program to terminate.
   //
   public BasicFrame()
   {
      setDefaultCloseOperation(DISPOSE_ON_CLOSE);
      addWindowListener(new WindowAdapter()
      {
         public void windowClosing(WindowEvent e)
         {
            System.exit(0);
         }
      } );
      setSize(mWidth, mHeight);
      setTitle(getClass().getName());

      Container contentPane = getContentPane();

      contentPane.addMouseMotionListener(new MouseMotionAdapter()
      {
         public void mouseMoved(MouseEvent e)
         {
            BasicFrameMouseMoved(e);
            repaint();
         }
      });
   }

   //...........................................................................
   //
   // Description: The BasicFrameMenu method is a procedure for building
   //              top level menus for Java applications.
   //
   public static JMenu BasicFrameMenu(
                                       Object   parent, // in: parent of menu new object
                                       Object[] items,  // in: items in new menu object
                                       Object   target  // in: object menu is attached to
                                     )
   {
      JMenu       m  = null;       // new menu object being created
      ButtonGroup bg = null;       // new menu items

      // Create new menu item
      if(parent instanceof JMenu) m = (JMenu) parent;
      else if(parent instanceof String) m = new JMenu((String) parent);
      else return null;

      // Go through input menu list, creating each item in menu
      for(int i = 0; i < items.length; i++)
      {
         if(items[i] instanceof String)  // standard menu item
         {
            JMenuItem mi = new JMenuItem((String) items[i]);
            if(target instanceof ActionListener) mi.addActionListener((ActionListener) target);
            m.add(mi);
         }
         else if(items[i] instanceof JCheckBoxMenuItem)  // check box item
         {
            JCheckBoxMenuItem cmi = (JCheckBoxMenuItem) items[i];
            if(target instanceof ItemListener) cmi.addItemListener((ItemListener) target);
            if(i == 0)
            {
               cmi.setState(true);
               bg = new ButtonGroup();
            }
            bg.add(cmi);
            m.add(cmi);
         }
         else if(items[i] == null) m.addSeparator();    // separator
      }
      return(m);
   }
//..............................................................................
// Class Members
//
   private static int mWidth  = 300;    // default window width
   private static int mHeight = 200;    // default window height
}
//..............................................................................
//  * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................



