//..............................................................................
// -*- Mode: Java; -*-
//                          Package   : DataBrowser
// DataBrowser.java         Author    : Deanna Evans (dke),
//                                      Ed Kraemer (eck) and,
//                                      Mark Pumar (map)
//
//  This file is part of DataBrowser.
//
//  DataBrowser is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
//
//  Imports
import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.io.*;
import java.text.*;
import BasicFrame.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import DataSet.*;
import DataView.*;

import Data.*;               // The package containing our stubs.
import org.omg.CosNaming.*;  // Client will use the naming service.
import org.omg.CORBA.*;      // All CORBA applications need these classes.

//..............................................................................
//
public class DataBrowser extends BasicFrame implements ActionListener,
                                                       ItemListener,
                                                       TreeSelectionListener
{
   //...........................................................................
   //
   // Description: The DataBrowser constructor creates the main application
   //              window and waits and responds to user generated events and
   //              responses from the server (DataReader).  The application
   //              interface consists of a menu bar, a text box for setting the
   //              server IP address, a tree for holding the list of available
   //              dataset files sent by the server, buttons for selecting
   //              data at various resolutions, and a display area for viewing
   //              the data.
   //
   public DataBrowser()
   {
      // Call base class constructor
      super();
      Container contentPane = getContentPane();

      // Create the menu bar for the DataBrowser
      JMenuBar mbar = new JMenuBar();
      mbar.add(BasicFrameMenu("File",     new java.lang.Object[] {"Save As",null,"Exit"},this));
      mbar.add(BasicFrameMenu("File Type",new java.lang.Object[] {new JCheckBoxMenuItem("Data"),
                                                                  new JCheckBoxMenuItem("Image")},this));
      mbar.add(BasicFrameMenu("Data",     new java.lang.Object[] {"Info..."},this));
      mbar.add(BasicFrameMenu("Help",     new java.lang.Object[] {"About..."},this));
      setJMenuBar(mbar);

      // Create the IP Address text area and connect button
      JPanel mIPPanel = new JPanel();
      mTextBox        = new JTextField("localhost",25);
      JLabel mIPLabel = new JLabel("IP Address");
      mConnectButton  = new JButton("Connect");
      mIPPanel.add(mIPLabel);
      mIPPanel.add(mTextBox);
      mIPPanel.add(mConnectButton);
      mConnectButton.addActionListener(this);

      // Create the area for the filename and directory structure and resolution buttons
      JPanel pp = new JPanel();
      pp.setLayout(new GridLayout(2,1));

      // Create the directory tree
      DefaultMutableTreeNode treeRoot = new DefaultMutableTreeNode("No Data File");
      mTree = new JTree( new DefaultTreeModel(treeRoot) );

      // Create a scroll pane to hold the tree
      mTreeScrollPane = new JScrollPane();
      mTreeScrollPane.setBorder(BorderFactory.createEtchedBorder());
      mTreeScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
      mTreeScrollPane.setViewportView(mTree);

      // Create a button panel for the resolutions
      JPanel buttonPanel = new JPanel();
      buttonPanel.setLayout(new GridLayout(4,1));
      buttonPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
      mButton1 = new JButton("");
      mButton1.addActionListener(this);
      mButton1.setBorder(BorderFactory.createRaisedBevelBorder());
      mButton2 = new JButton("");
      mButton2.addActionListener(this);
      mButton2.setBorder(BorderFactory.createRaisedBevelBorder());
      mButton3 = new JButton("");
      mButton3.addActionListener(this);
      mButton3.setBorder(BorderFactory.createRaisedBevelBorder());
      mButton4 = new JButton("");
      mButton4.addActionListener(this);
      mButton4.setBorder(BorderFactory.createRaisedBevelBorder());
      buttonPanel.add(mButton1);
      buttonPanel.add(mButton2);
      buttonPanel.add(mButton3);
      buttonPanel.add(mButton4);

      pp.add(mTreeScrollPane);
      pp.add(buttonPanel);

      // Create the status bar
      JPanel status = new JPanel();
      mPl = new JPanel();
      mPr = new JPanel();
      mPl.setBackground(Color.gray);
      mPr.setBackground(Color.gray);
      mMessage = new JLabel("DataBrowser            ");
      mCoords  = new JLabel("coords");
      status.setBackground(Color.gray);
      status.setBorder(BorderFactory.createEtchedBorder());
      status.setLayout(new FlowLayout(FlowLayout.LEFT,16,0));
      mPl.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));
      mPr.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));
      mPl.add(mMessage);
      mPr.add(mCoords);
      mMessage.setHorizontalAlignment(JLabel.LEFT);
      mCoords.setHorizontalAlignment(JLabel.LEFT);
      status.add(mPl,BorderLayout.WEST);
      status.add(mPr,BorderLayout.EAST);

      mSp = new JScrollPane();
      appAddData(null);

      // Add the panels to create the GUI
      contentPane.add(mIPPanel,BorderLayout.NORTH);
      contentPane.add(status,BorderLayout.SOUTH);
      contentPane.add(pp,BorderLayout.WEST);
      contentPane.add(mSp,BorderLayout.CENTER);

      setResizable(false);

      repaint();
   }
   //...........................................................................
   //
   // Description:  Converts a point pointed to by the mouse into window
   //               coordinates for display at the bottom of the GUI
   //
   private void appConvertPointToScrollPane(
                                             Point pt  // in|out: point for conversion
                                            )
   {
      // Convert point from screen coordinates to window coordinates
      final Container contentPane = getContentPane();
      SwingUtilities.convertPointToScreen(pt,contentPane);
      SwingUtilities.convertPointFromScreen(pt,mSp);
   }
   //...........................................................................
   //
   // Description:  Converts a point pointed to by the mouse into data
   //               coordinates for display at the bottom of the GUI
   //
   private void appConvertPointToData(
                                       Point pt  // in|out: point for conversion
                                     )
   {
      // Convert point to data coordinates
      final Container contentPane = getContentPane();
      SwingUtilities.convertPointToScreen(pt,contentPane);
      SwingUtilities.convertPointFromScreen(pt,mD);
   }
   //...........................................................................
   //
   // Description:  Mouse event listener. Re-implemented from base class.
   //               Captures mouse moved events.
   //
   public void BasicFrameMouseMoved(
                                     MouseEvent e  // in: mouse event
                                   )
   {
      String    loc         = "";                 // display string
      Container contentPane = getContentPane();   // content pane
      Point     pt1         = e.getPoint();       // mouse location
      Point     pt2         = e.getPoint();       // mouse location

      // Convert mouse location to data and window coordinates
      appConvertPointToData(pt1);
      appConvertPointToScrollPane(pt2);

      // Determine if the mouse is located inside the data display
      if((mD.dataSetInside(pt1)   == 0) || (pt2.x < 0))
      {
         // The mouse is outside the data display
         // Change cursor to default cursor
         contentPane.setCursor(Cursor.getDefaultCursor());
      }
      else
      {
         // The mouse is inside the data display
         // Change cursor to cross-hair
         contentPane.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
         loc = mD.dataSetToLocation(pt1);
      }

      // Set the coordinates of the current mouse location on the bottom of the GUI
      mCoords.setText(loc);
   }
   //...........................................................................
   //
   // Description:  Get the preferred size of the DataBrowser
   //
   public Dimension getPreferredSize()
   {
      // return preferred application window dimensions
      return new Dimension(mWidth,mHeight);
   }
   //...........................................................................
   //
   // Description:  Displays the data
   //
   public void appAddData(
                           byte[] data  // in: byte array containing the data
                         )
   {
      // Get display infrastucture sizes
      int top   = getInsets().top+1;                                    // frame size
      int rgt   = getInsets().right;                                    // frame size
      int dx    = mSp.getVerticalScrollBar().getPreferredSize().width;  // SB width
      int dy    = mSp.getVerticalScrollBar().getPreferredSize().width;  // SB width
      int dpane = mTree.getPreferredSize().width;                       // frame size
      int wtm   = mCoords.getPreferredSize().height + getJMenuBar().getSize().height;

      // If data is null, display default window, otherwise display data
      if(data == null) mD = new DataView(mDefaultWidth,mDefaultHeight);
      else mD = new DataView(data);

      // Create the image for display
      mD.dataSetImageCreate();

      // Get the current dimensions and resize the GUI window as appropriate
      mSp.setViewportView(mD);
      mSp.setSize(mD.dataSetWidth(),mD.dataSetHeight());
      Rectangle r = appWindowSize(mD.dataSetWidth()-1, mD.dataSetHeight()-1);
      mWidth  = 1+r.width+getInsets().left+getInsets().right+dx+dpane;
      mHeight = 1+r.height+getInsets().top+getInsets().bottom+wtm+dy;
      setSize(mWidth,mHeight);

      // Repaint window
      pack();
      repaint();
   }
   //...........................................................................
   //
   // Description:  The actionPerformed method processes all the major events
   //               for the DataBrowser application.  This includes menus as
   //               well as button events.
   //
   public void actionPerformed(
                                ActionEvent evt  // in: event information
                              )
   {
      // See if this action is for the menu
      if (evt.getSource() instanceof JMenuItem)
      {
         // Determine which menu was selected
         JMenuItem mi = (JMenuItem)evt.getSource();
         String str   = mi.getText();
         if(str.equals("Exit")) System.exit(0);
         else if(str.equals("Save As"))
         {
            // Create and display a new file dialog
            FileDialog fd = new FileDialog(this,"Save Data File",FileDialog.SAVE);
            fd.setDirectory(".");
            fd.show();

            // Determine if file selected is valid
            if(fd.getFile() != null)
            {
               String fileName = fd.getDirectory()+fd.getFile();
               mMessage.setText("Writing...      ");
               mMessage.paintImmediately(mMessage.getBounds());
               mD.dataSetWrite(fileName);
               mMessage.setText("Done            ");
            }
         }
         else if(str.equals("Info..."))
         {
            java.lang.Object[] path = null;

            // Determine if a valid file is pointed to in the directory tree
            if (mTree.getSelectionPath() != null)
            {
               // Determine the file that is pointed to
               String filename = "";
               path = mTree.getSelectionPath().getPath();
               if (path != null) filename = path[path.length-1].toString();

               // Obtain data information
               String info = "File: " + filename +"\n"
                           + "Dimensions: " + mD.dataSetWidth() + " X " + mD.dataSetHeight() + "\n"
                           + "X-Origin: " + mD.dataSetXOrgin() + " Y-Origin: " + mD.dataSetYOrgin() + "\n"
                           + "Spacing: " + mD.dataSetXRes() + " X " + mD.dataSetYRes() + "\n";

               // Display the information for this file
               JOptionPane.showMessageDialog(this,
                                             info,
                                             "Data Information",
                                             JOptionPane.INFORMATION_MESSAGE);
            }
            else
            {
               // No valid tree or file were found, display default information
               JOptionPane.showMessageDialog(this,
                                             "No data loaded",
                                             "Data Information",
                                             JOptionPane.INFORMATION_MESSAGE);
            }
         }
         else if(str.equals("About..."))
         {
            // Display information about the DataBrowser
            JOptionPane.showMessageDialog(this,
                                          "DataBrowser Version 1.0\n Copyright (C) 2000 DKE, ECK, MAP\n"+
                                          "DataBrowser comes with ABSOLUTELY NO WARRANTY,\n"+
                                          "is free software, and you are welcome to redistribute it\n"+
                                          "under certain conditions (see GNU Public License for details).",
                                          "About DataBrowser",
                                          JOptionPane.INFORMATION_MESSAGE);
        }
      } // end of if for JMenuItem; begin button event handling
      else if (evt.getSource() == mConnectButton)  // connect button pushed
      {
         // Reset plan view map to grey screen
         appAddData(null);

         // Reset tree file path
         mTreeFilePath = "";

         // Reset button labels.
         mButton1.setText("");
         mButton2.setText("");
         mButton3.setText("");
         mButton4.setText("");

         // The Connect button has been pressed
         // Set the IP Address in the text box
         mIPAddress[3] = mTextBox.getText().trim();

         // Get Path Delimiter from the server for constructing file names
         getPathDelimiter();

         // Create the root
         DefaultMutableTreeNode treeRoot  = new DefaultMutableTreeNode(mIPAddress[3]);

         // Read the nodes and build the tree
         nextTreeNode(treeRoot, 1, ".");

         // Create the tree
         mTree = new JTree( new DefaultTreeModel(treeRoot) );
         mTree.setRootVisible(false);

         // Add a listener to the tree
         mTree.addTreeSelectionListener(this);

         // Place the tree in the scroll window
         mTreeScrollPane.setViewportView(mTree);
         repaint();
      }
      else if (evt.getSource() == mButton1)  // full resolution button pushed
      {
         // Determine if the button label has been set
         if (!mButton1.getText().equals(""))
         {
            // The button label is set, load data at selected resolution
            getData(mTreeFilePath, 1);
         }
      }
      else if (evt.getSource() == mButton2)   // low-resoliton button pushed
      {
         // Determine if the button label has been set
         if (!mButton1.getText().equals(""))
         {
            // The button label is set, load data at selected resolution
            getData(mTreeFilePath, mFactor[0]);
         }
      }
      else if (evt.getSource() == mButton3)  // lower-resolution button pushed
      {
         // Determine if the button label has been set
         if (!mButton1.getText().equals(""))
         {
            // The button label is set, load data at selected resolution
            getData(mTreeFilePath,mFactor[1]);
         }
      }
      else if (evt.getSource() == mButton4)   // lowest-resolution button pushed
      {
          // Determine if the button label has been set
        if (!mButton1.getText().equals(""))
         {
            // The button label is set, load data at selected resolution
            getData(mTreeFilePath, mFactor[2]);
         }
      }
   }
   //...........................................................................
   //
   // Description:  The itemStateChanged method is a check box item listener
   //               and is used to determine if the DataBrowser should request
   //               data or imagery from the server.
   //
   public void itemStateChanged(
                                 ItemEvent evt  // in: event information
                               )
   {
      // The file data type has changed, set the global variable
      JCheckBoxMenuItem cmi = (JCheckBoxMenuItem) evt.getSource();
      mLoadFileAs = cmi.getText();
   }
   //...........................................................................
   //
   // Description:  The appWindowSize method determines the proper window size
   //               based on the data dimension
   //
   public Rectangle appWindowSize(
                                   int width, // in: data width
                                   int height // in: data height
                                 )
   {
      Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
      int size = (width < height ? width : height);
      Rectangle r;

      // Create a rectangle based on the data size
      if((size >= mWindowSize[4]) && (d.width >= mWindowSize[5]))
       {r = new Rectangle(mWindowSize[4], mWindowSize[4]); }
      else if((size >= mWindowSize[2]) && (d.width >= mWindowSize[3]))
       {r = new Rectangle(mWindowSize[2]+25,  mWindowSize[2]+25); }
      else if((size >= mWindowSize[1]) && (d.width >= mWindowSize[2]))
       {r = new Rectangle(mWindowSize[1]+25,  mWindowSize[1]+25); }
      else
       {r = new Rectangle(mWindowSize[0]+25,  mWindowSize[0]+25); }

      return(r);
   }
   //...........................................................................
   //
   // Description:  The getData method gets the data in the requested format
   //               and resolution.
   //
   public void getData(
                        String fileName,   // in: name of file containing data
                        int    resolution  // in: requested data resolution
                       )
   {
      try
      {
         // Create and initialize the ORB
         ORB orb = ORB.init(mIPAddress, null);

         // Get the root naming context
         org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");
         NamingContext ncRef = NamingContextHelper.narrow(objRef);

         // Resolve the object reference in naming
         DataInfo dataRef = DataInfoHelper.narrow(ncRef.resolve(mPath));

         // Call the server object and print results
         System.out.println("Getting Data...");

         Data.UnboundedDataHolder sa = new Data.UnboundedDataHolder();

         // Get the data from the server
         int val = dataRef.getData(fileName, resolution, mLoadFileAs, sa);

         // Display an error dialog box if the getData function failed
         if (val == 0)  JOptionPane.showMessageDialog(this,
                                                      "Get Data Failed",
                                                      "Error",
                                                      JOptionPane.ERROR_MESSAGE);
         else
         {
            // Data returned successfully, display it
            appAddData(sa.value);
         }
      }
      catch(Exception e)
      {
         JOptionPane.showMessageDialog(this,
                                       "Unable to retrieve data from file "+fileName,
                                       "Error",
                                       JOptionPane.ERROR_MESSAGE);
      }
   }
   //...........................................................................
   //
   // Description:  The getDataLocation method gets the current directory
   //               location of the available datasets from server
   //
   public String getDataLocation()
   {
      try
      {
         // Create and initialize the ORB
         ORB orb = ORB.init(mIPAddress, null);

         // Get the root naming context
         org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");
         NamingContext ncRef = NamingContextHelper.narrow(objRef);

         // Resolve the object reference in naming
         DataInfo dataRef = DataInfoHelper.narrow(ncRef.resolve(mPath));

         // Call the server object and print results
         System.out.println("Getting Data Location...");

         StringHolder dir = new StringHolder(".");
         int val = dataRef.getLocation(dir);

         // Display an error dialog box if the getLocation function failed
         if (val == 0)  JOptionPane.showMessageDialog(this,
                                                      "Get Data Location Failed",
                                                      "Error",
                                                      JOptionPane.ERROR_MESSAGE);

         return(dir.value);
      }
      catch(Exception e)
      {
         JOptionPane.showMessageDialog(this,
                                       "Unable to obtain location of data file(s)",
                                       "Error",
                                       JOptionPane.ERROR_MESSAGE);
         return(".");
      }
   }
   //...........................................................................
   //
   // Description:  The getDataFiles method gets the current list of available
   //               data files from the server for display
   //
   public String getDataFiles(
                               String dir // in: location of data files
                             )
   {
      try
      {
         // Create and initialize the ORB
         ORB orb = ORB.init(mIPAddress, null);

         // Get the root naming context
         org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");
         NamingContext ncRef = NamingContextHelper.narrow(objRef);

         // Resolve the object reference in naming
         DataInfo dataRef = DataInfoHelper.narrow(ncRef.resolve(mPath));

         // Call the server object and print results
         System.out.println("Getting Data Files...");

         StringHolder files = new StringHolder("");
         int val = dataRef.getFiles(dir,files);

         // Display an error dialog if the getFiles function failed
         if (val == 0)  JOptionPane.showMessageDialog(this,
                                                      "Get Data Files Failed",
                                                      "Error",
                                                      JOptionPane.ERROR_MESSAGE);
         return(files.value);
      }
      catch(Exception e)
      {
         JOptionPane.showMessageDialog(this,
                                       "No data abailable at location "+dir,
                                       "Error",
                                       JOptionPane.ERROR_MESSAGE);

         return("");
      }
   }
   //...........................................................................
   //
   // Description:  The getFileResolution methods gets the file resolution
   //               of the file selected by the user.
   //
   public int [] getFileResolution(
                                    String file  // in: selected file name
                                   )
   {
      try
      {
         // Create and initialize the ORB
         ORB orb = ORB.init(mIPAddress, null);

         // Get the root naming context
         org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");
         NamingContext ncRef = NamingContextHelper.narrow(objRef);

         // Resolve the object reference in naming
         DataInfo dataRef = DataInfoHelper.narrow(ncRef.resolve(mPath));

         // Call the server object and print results
         System.out.println("Getting File Resolution...");

         IntHolder width  = new IntHolder(0);
         IntHolder height = new IntHolder(0);
         IntHolder length = new IntHolder(0);

         int size = dataRef.getFileResolution(file,width,height,length);
         int [] resolution = new int[4];
         resolution[0] = size;
         resolution[1] = width.value;
         resolution[2] = height.value;
         resolution[3] = length.value;

         return(resolution);
      }
      catch(Exception e)
      {
         JOptionPane.showMessageDialog(this,
                                       "Unable to get information for file "+file,
                                       "Error",
                                       JOptionPane.ERROR_MESSAGE);
         int [] resolution = new int[3];
         resolution[0] = resolution[1] = resolution[2] = 0;
         return(resolution);
      }
   }
   //...........................................................................
   //
   // Description:  The getPathDelimiter method gets the path delimiter used
   //               by the server.
   //
   public void getPathDelimiter()
   {
      try
      {
         // Create and initialize the ORB
         ORB orb = ORB.init(mIPAddress, null);;

         // Get the root naming context
         org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");
         NamingContext ncRef = NamingContextHelper.narrow(objRef);

         // Resolve the object reference in naming
         DataInfo dataRef = DataInfoHelper.narrow(ncRef.resolve(mPath));

         // Call the server object and print results
         System.out.println("Getting Path Delimiter...");


         StringHolder pathDel = new StringHolder(File.separator);
         int val = dataRef.getPathDelimiter(pathDel);

         // Display an error dialog if the getPathDelimiter function failed
         if (val == 0)  JOptionPane.showMessageDialog(this,
                                                      "Get Path Delimiter Failed",
                                                      "Error",
                                                      JOptionPane.ERROR_MESSAGE);

         mPathDel = pathDel.value;
      }
      catch(Exception e)
      {
         JOptionPane.showMessageDialog(this,
                                       "Unable to obtain file path inforamtion",
                                       "Error",
                                       JOptionPane.ERROR_MESSAGE);
      }
   }

   //...........................................................................
   //
   // Description:  The nextTreeNode method is used to retrieve the files names
   //               and directories in the tree in the left portion of the main
   //               window.  It is recursive.
   //
   private void nextTreeNode(
                              DefaultMutableTreeNode parent,  // in: parent node
                              int                    depth,   // in: branch depth
                              String                 pathname // in: path name
                             )
   {
      DefaultMutableTreeNode child;           // parent's children
      int                    index;           // index to tab in filelist

      // Get the data files available from the server
      String filelist = getDataFiles(pathname);

      if (filelist.equals(""))
      {
         // No files or subdirectories were found for this node
         // Create node as a child of the parent
         child = new DefaultMutableTreeNode("No Data File");

         // Add the child to the parent
         parent.add(child);
         return;
      }

      for (int loop = 0; loop < filelist.length(); loop = index + 1)
      {
         // Loop through the list of files
         index = filelist.indexOf("\t", loop);
         String name = filelist.substring(loop, index);

         if (name.startsWith("."+mPathDel))
         {
            // This is a directory and needs recursive call to have
            // it's files and directories added to the tree

            // Create a node as a child of the parent
            child = new DefaultMutableTreeNode(name.substring(2));

            // Call this function to add its children
            nextTreeNode(child, depth+1, pathname + mPathDel + name.substring(2));

            // Add the child to the parent
            parent.add(child);
         }
         else
         {
            // This is a file and needs to be added to the tree
            // Create the node as a child of the parent
            child = new DefaultMutableTreeNode(name);

            // Add the child to the parent
            parent.add(child);
         }
      }
   }
   //...........................................................................
   //
   // Description:  The valueChanged method is a callback for tree actions.
   //
   public void valueChanged(
                             TreeSelectionEvent evt  // in: tree event information
                           )
   {
      // Reset data display
      appAddData(null);

      // Determine if the event occured due to file or folder.
      TreeModel model = mTree.getModel();

      if (model.isLeaf(mTree.getLastSelectedPathComponent()))
      {
         // This is a file.  Proceed to get the tree path
         TreePath selPath = mTree.getSelectionPath();
         String filename = selPath.getLastPathComponent().toString();

         // Determine if this is the default no file name
         if (filename.equals("No Data File"))
         {
            // This is not a data file, so return
            // Reset button labels
            mButton1.setText("");
            mButton2.setText("");
            mButton3.setText("");
            mButton4.setText("");
            return;
         }

         // Create the path to the file as a string
         mTreeFilePath = "." + mPathDel;
         java.lang.Object[] path = selPath.getPath();

         for (int loop = 0; loop < path.length; loop++)
         {
            if (!path[loop].toString().equals(mIPAddress[3]))
            {
               // This is not equal to the root string, so add the name to the
               // file path for this node.
               mTreeFilePath = mTreeFilePath + path[loop].toString() + mPathDel;
            }
         }
         mTreeFilePath = mTreeFilePath.substring(0, mTreeFilePath.lastIndexOf(mPathDel));

         // Load file resolution information
         int [] default_res = getFileResolution(mTreeFilePath);

         int iXdata = default_res[1] - 1;
         int iYdata = default_res[2] - 1;
         int hsize  = default_res[3];
         float fsize = default_res[0];

         // Determine the number of points and data size
         int number_points = iXdata * iYdata;
         int data_size = (int)(fsize - hsize)/number_points;

         String[]     res  = new String[4];
         NumberFormat nf   = NumberFormat.getNumberInstance();
         nf.setMaximumFractionDigits(1);

         res[0] = iXdata + " x " + iYdata + " (" + nf.format(fsize/1000.0) + " KB)";

         int array_position = 0;

         // Need to determine the possible sizes for the file.
         // First get the the smallest dimension.
         int min_dimension = 0;
         int max_dimension = 0;
         if (iXdata < iYdata) {min_dimension = iXdata; max_dimension = iYdata;}
         else {min_dimension = iYdata; max_dimension = iXdata;}

         // Now determine the factors for the smallest dimension and
         // see if the result is a factor of the other dimension
         for (int loop = 2; loop < min_dimension; loop++)
         {
            if(min_dimension%loop == 0)
            {
               // Found factor of the minimum dimension, now
               // need to see if it is a factor of the maximum dimension
               if(max_dimension%loop == 0)
               {
                  mFactor[array_position] = loop;
                  array_position++;
                 if (array_position == 3) break;
               }
            }
         }
         if (array_position != 3)
         {
            for (int loop = array_position; loop < 3; loop++)
            {
               mFactor[loop] = 0;
            }
         }

         // Determine the size of the file for the resolutions
         double filesize1 = 0;
         double filesize2 = 0;
         double filesize3 = 0;

         if (mFactor[0] != 0) filesize1 = (((double)iXdata/(double)mFactor[0] + 1) *
                                          ((double)iYdata/(double)mFactor[0] + 1)) * (double)data_size
                                          + hsize;
         if (mFactor[1] != 0) filesize2 = (((double)iXdata/(double)mFactor[1] + 1) *
                                          ((double)iYdata/(double)mFactor[1] + 1)) * (double)data_size
                                          + hsize;
         if (mFactor[2] != 0) filesize3 = (((double)iXdata/(double)mFactor[2] + 1) *
                                          ((double)iYdata/(double)mFactor[2] + 1)) * (double)data_size
                                          + hsize;

         if (mFactor[0] != 0) res[1] = iXdata/mFactor[0] + " x " + iYdata/mFactor[0]
                                                         + " (" + nf.format(filesize1/1000.0) + " KB)";
         else res[1] = " ";
         if (mFactor[1] != 0) res[2] = iXdata/mFactor[1] + " x " + iYdata/mFactor[1]
                                                                 + " (" + nf.format(filesize2/1000.0) + " KB)";
         else res[2] = " ";
         if (mFactor[2] != 0) res[3] = iXdata/mFactor[2] + " x " + iYdata/mFactor[2]
                                                         + " (" + nf.format(filesize3/1000.0) + " KB)";
         else res[3] = " ";

         // Set the buttons with the resolutions computed
         mButton1.setText(res[0]);
         mButton2.setText(res[1]);
         mButton3.setText(res[2]);
         mButton4.setText(res[3]);
      }
   }
   //...........................................................................
   //
   // Description:  Main for DataBrowser
   //
   public static void main(String[] args)
   {
      JFrame f = new DataBrowser();
      f.setVisible(true);
   }
   //...........................................................................
   // Class Data Members
   //
   private JScrollPane mSp;                             // panel for main view
   private JTextField  mTextBox;                        // text box for IP Address
   private JButton     mConnectButton;                  // button for IP address connection
   private JScrollPane mTreeScrollPane;                 // scroll pane for tree
   private JTree       mTree;                           // tree
   private JButton     mButton1;                        // button for resolution choice 1
   private JButton     mButton2;                        // button for resolution choice 2
   private JButton     mButton3;                        // button for resolution choice 3
   private JButton     mButton4;                        // button for resolution choice 4
   private JPanel      mPl;                             // panel to hold message text
   private JPanel      mPr;                             // panel to hold corrdinate text
   private JLabel      mMessage;                        // label for messages
   private JLabel      mCoords;                         // label for coordinates
   private DataView    mD             = null;           // data
   private int         mMetric        = 1;              // distance, area, volume
   private int         mHeight        = 0;              // window frame width
   private int         mWidth         = 0;              // window frame height
   private int[]       mFactor        = new int[3];     // factors for file resolution
   private String      mPathDel       = File.separator; // path delimeter
   private String      mTreeFilePath  = "";             // file path for tree
   private String      mLoadFileAs    = "Data";         // data type to load file as
   private String      mIPAddress[]   = {"-ORBInitialPort", DataView.mPORT, "-ORBInitialHost", DataView.mHOST};
   //
   // Static Data Members
   //
   private static NameComponent[] mPath = { new NameComponent(DataView.mCONTEXT_ID, DataView.mCONTEXT_KIND),
                                            new NameComponent(DataView.mOBJECT_ID,  DataView.mOBJECT_KIND) };
   private static int    mDefaultWidth  = 401;          // default width of data display
   private static int    mDefaultHeight = 401;          // default height of data display
   private static int [] mWindowSize    = {300,400,600,900,1200,1800}; // window sizes.

}
//..............................................................................
//  * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................
