//..............................................................................
// -*- Mode: Java; -*-
//                          Package   : DataBrowser
// DataSet.java             Author    : Deanna Evans (dke),
//                                      Ed Kraemer (eck) and,
//                                      Mark Pumar (map)
//
//  This file is part of DataBrowser.
//
//  DataBrowser is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
//
package DataSet;
// Imports
import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.text.*;
import java.lang.*;
//
//.............................................................................
//
public class DataSet extends JPanel
{
   //..........................................................................
   //
   // Description:  The DataSet default constructor creates a default dataset
   //               and image when the browser starts or is cleared.
   //
   public DataSet(
                   int Nx,   // in: image/data width
                   int Ny    // in: image/data height
                 )
   {
      int i = 0;             // counter
      int j = 0;             // counter

      // Initialize data members
      mN       = 16;                    // header length in bytes/4 (int)
      mType    = mDATA;                 // image or data
      mKind    = mINT;                  // integer or real
      mSize    = 1;                     // data size (1, 2, 4, or 8 bytes)
      mNx      = Nx;                    // image width
      mNy      = Ny;                    // image height
      mOrgX    = 0;                     // image x-origin
      mOrgY    = 0;                     // image y-origin
      mResX    = 1;                     // image x-resolution
      mResY    = 1;                     // image y-resolution
      mDataMin = 0;                     // data min value
      mDataMax = 0;                     // data max value
      mDc      = 1;                     // contour spacing
      mConMin  = 0;                     // min contour value
      mLocMin  = new Point(0,0);        // location of min
      mLocMax  = new Point(0,0);        // location of max

      // Build Header
      mHead = new int[mN];

      mHead[0]  = mN;
      mHead[1]  = mType;
      mHead[2]  = mKind;
      mHead[3]  = mSize;
      mHead[4]  = mNx;
      mHead[5]  = mNy;
      mHead[6]  = mOrgX;
      mHead[7]  = mOrgY;
      mHead[8]  = mResX;
      mHead[9]  = mResY;
      mHead[10] = 0;
      mHead[11] = 0;
      mHead[12] = 0;
      mHead[13] = 0;
      mHead[14] = 0;
      mHead[15] = 0;

      // Create Data
      mByteData   = new byte[mNx][mNy];

      // Initialize data to zero
      for(i = 0; i < mNx; i++)
      {
         for(j = 0; j < mNy; j++)
         {
            mByteData[i][j] = 0;
         }
      }
   }
   //..........................................................................
   //
   // Description:  The DataSet constructor creates an image from the input
   //               byte array obtained from the server.
   //
   public DataSet(
                   byte[] data   // in: data for image construction
                 )
   {
      if (data != null)
      {
         int    i    = 0;        // counter
         int    j    = 0;        // counter
         double D    = 0;        // countour spacing
         double p    = 0;        // countour scale factor

         try
         {
            // Create stream for "reading" data
            ByteArrayInputStream bin  = new ByteArrayInputStream(data);
            DataInputStream din       = new DataInputStream(bin);

            // Read header
            mN    = din.readInt();
            mHead = new int[mN];
            mHead[0] = mN;
            for(j = 1; j < mN; j++) mHead[j]= din.readInt();

            // Store first part of header in class members
            mType = mHead[1];     // image or data
            mKind = mHead[2];     // integer or real
            mSize = mHead[3];     // data size (1, 2, 4, or 8 bytes)
            mNx   = mHead[4];     // data width
            mNy   = mHead[5];     // data height
            mOrgX = mHead[6];     // data location   (x)
            mOrgY = mHead[7];     // data location   (y)
            mResX = mHead[8];     // data resolution (x)
            mResY = mHead[9];     // data resolution (y)

            // Initialize location of min and max
            mLocMin = new Point(0,0);
            mLocMax = new Point(0,0);

            // Allocate data storage space based on kind and size of data
            if(mKind ==  mINT)   // integer data
            {
               if(mSize == 1) mByteData = new byte[mNx][mNy];
               else if(mSize == 2) mShortData = new short[mNx][mNy];
               else if(mSize == 4) mIntData   = new int[mNx][mNy];
               else
               {
                  din.close();
                  return;
               }
            }
            else if(mKind == mREAL)   // real data
            {
               if(mSize == 4) mFloatData = new float[mNx][mNy];
               else if(mSize == 8) mDoubleData = new double[mNx][mNy];
               else
               {
                  din.close();
                  return;
               }
            }
            else
            {
               din.close();
               return;
            }

            // read data
            for(i = 0; i < mNx; i++)
            {
               for(j = 0; j < mNy; j++)
               {
                  if(mByteData != null)
                  {
                     mByteData[i][j] = din.readByte();
                     if(mDataMin > mByteData[i][j]) {mDataMin = mByteData[i][j]; mLocMin.x = i; mLocMin.y = j;}
                     if(mDataMax < mByteData[i][j]) {mDataMax = mByteData[i][j]; mLocMax.x = i; mLocMax.y = j;}
                  }
                  else if(mShortData  != null)
                  {
                     mShortData[i][j]  = din.readShort();
                     if(mDataMin > mShortData[i][j]) {mDataMin = mShortData[i][j]; mLocMin.x = i; mLocMin.y = j;}
                     if(mDataMax < mShortData[i][j]) {mDataMax = mShortData[i][j]; mLocMax.x = i; mLocMax.y = j;}
                  }
                  else if(mIntData    != null)
                  {
                     mIntData[i][j]    = din.readInt();
                     if (mDataMin > mIntData[i][j]) {mDataMin = mIntData[i][j]; mLocMin.x = i; mLocMin.y = j;}
                     if (mDataMax < mIntData[i][j]) {mDataMax = mIntData[i][j]; mLocMax.x = i; mLocMax.y = j;}
                  }
                  else if(mFloatData  != null)
                  {
                     mFloatData[i][j]  = din.readFloat();
                     if(mDataMin > mFloatData[i][j]) {mDataMin = mFloatData[i][j]; mLocMin.x = i; mLocMin.y = j;}
                     if(mDataMax < mFloatData[i][j]) {mDataMax = mFloatData[i][j]; mLocMax.x = i; mLocMax.y = j;}
                  }
                  else if(mDoubleData != null)
                  {
                     mDoubleData[i][j] = din.readDouble();
                     if(mDataMin > mDoubleData[i][j]) {mDataMin = mDoubleData[i][j]; mLocMin.x = i; mLocMin.y = j;}
                     if(mDataMax < mDoubleData[i][j]) {mDataMax = mDoubleData[i][j]; mLocMax.x = i; mLocMax.y = j;}
                  }
                  else
                  {
                     din.close();
                     return;
                  }
               }
            }
            din.close();
         }
         catch(IOException e){ }

         // compute contour factor
         if (mType == mIMAGE) {mDataMin = mHead[12]; mDataMax = mHead[13];}
         D = (mDataMax - mDataMin)/mNumCon;
         if(D <= 0) mDc = 0;
         else
         {
            p = (int) (Math.log(D)/Math.log(10.0));
            mDc = (int) (D/Math.pow(10.0,p) + 1.0);
            mDc = mDc*Math.pow(10.0,p);
            mConMin = (int) (mDataMin/mDc);
            mConMin = mConMin*mDc;
         }
      }
   }
   //..........................................................................
   //
   // Description:  The dataSetImageCreate method creates a color contour
   //               image of the dataset.
   //
   public void dataSetImageCreate()
   {
      int   i, j, k;              // counters
      int   imgVal;               // image brightness value
      int   red;                  // red componet of image
      int   green;                // green component of image
      int   blue;                 // blue component of image
      int   gamma;                // image gamma
      int   counter = 0;          // array index

      // Allocate memory to store pixels
      mPixels = new int[mNx*mNy];

      // Compute contours and color map
      if(mType == mDATA)
      {
         for(i = 0; i < mNx; i++)
         {
            for(j = 0; j < mNy; j++)
            {
               if(mType == mIMAGE)  // data is already imagery
               {
                  if(mByteData != null)   // byte: grey scale image
                  {
                     red =  green = blue = mByteData[i][j];
                     gamma = 255 - mByteData[i][j];
                     imgVal = (gamma << 24) + (red << 16) + (green << 8) + blue;
                  }
                  else if(mShortData  != null)  // short: extract color bits
                  {
                     red   = mShortData[i][j] & 0xfc00;  // top six bits
                     green = mShortData[i][j] & 0x03e0;  // middle 5 bits
                     blue  = mShortData[i][j] & 0x001f;  // bottom 5 bits
                     gamma = 255;
                     imgVal = (gamma << 24) + (red << 16) + (green << 8) + blue;
                  }
                  else if(mIntData    != null) imgVal = mIntData[i][j];  // int: full color image
                  else imgVal = 0;
               }
               else  // data is not image; contruct color contour
               {
                  k = dataSetImageContour(i, j);   // get contour number
                  red   = mINTENSITY*mRed[k];      // get color components
                  green = mINTENSITY*mGreen[k];
                  blue  = mINTENSITY*mBlue[k];
                  gamma = 255 - mINTENSITY;
                  imgVal = (gamma << 24) + (red << 16) + (green << 8) + blue;
               }
               mPixels[i + mNx*(mNy-j-1)] = imgVal;    // put image value into image array
            } // loop on j
         } // loop on i
      } // if data
      else if(mType == mIMAGE && mIntData != null)  //  data is image; load
      {
         for(i = 0; i < mNx; i++)
         {
            for(j = 0; j < mNy; j++)
            {
               mPixels[counter] = mIntData[i][j];
               counter++;
            }
         }
      }

      // Create image and display
      mImage = createImage(new MemoryImageSource(mNx,mNy,mPixels,0,mNx));
      repaint();
   }
   //..........................................................................
   //
   // Description:  The dataSetImageContour method uses the data contour
   //               spacing, offset, and data value to determine the
   //               contour number.
   //
   private int dataSetImageContour(
                                    int i, // in: x data coordinate
                                    int j  // in: y data coordinate
                                  )
   {
      double data = dataSetValue(i, j);  // get data value at (i,j)
      int n       = mNumCon;             // contour number (return value)
      int k       = 0;                   // counter

      for(k = 0; k <= mNumCon; k++)
      {
         if(data <= k*mDc + mConMin)
         {
            n = k;
            break;
         }
      }
      return(n);
   }
   //..........................................................................
   //
   // Description: The dataSetValue method returns the data value at the
   //              indicated input location.
   //
   public double  dataSetValue(
                                int x, // in: x data coordinate
                                int y  // in: y data coordinate
                              )
   {
      double data = 0;
      int i = (x >= 0 ? x : 0); i = (i < mNx ? i : (mNx-1));  // check x location
      int j = (y >= 0 ? y : 0); j = (j < mNy ? j : (mNy-1));  // check y location
      if(mByteData != null) data = mByteData[i][j];           // set return value
      else if(mShortData  != null) data = mShortData[i][j];   // based on data
      else if(mIntData    != null) data = mIntData[i][j];     // type
      else if(mFloatData  != null) data = mFloatData[i][j];
      else if(mDoubleData != null) data = mDoubleData[i][j];
      else data = 0;

      return(data);    // return value at (x,y)
   }
   //..........................................................................
   //
   // Description: The dataSetHeight returns the dataset height
   //
   public int dataSetHeight()
   {
      return(mNy);
   }
   //..........................................................................
   //
   // Description: The dataSetWidth returns the dataset width
   //
   public int dataSetWidth()
   {
      return(mNx);
   }
   //..........................................................................
   //
   // Description: The dataSetXRes returns the dataset x-resolution
   //
   public int dataSetXRes()
   {
      return(mResX);
   }
   //..........................................................................
   //
   // Description: The dataSetYRes returns the dataset y-resolution
   //
   public int dataSetYRes()
   {
      return(mResY);
   }
   //..........................................................................
   //
   // Description:  The dataSetXOrgin returns the dataset x-origin
   //
   public int dataSetXOrgin()
   {
      return(mOrgX);
   }
   //..........................................................................
   //
   // Description: The dataSetYOrgin returns the dataset y-origin
   //
   public int dataSetYOrgin()
   {
      return(mOrgY);
   }
   //..........................................................................
   //
   // Description:  The dataSetInside method returns 1 if the input point is
   //               located within the dataset, zero otherwise.
   //
   public int dataSetInside(
                              Point grid  // in: point
                           )
   {
      // Invert y coordinate
      int y = mNy - 1 - grid.y;

      // Determine if point is within dataset
      if((grid.x >= 0) && (grid.x < mNx) && (y >= 0) && (y < mNy)) return(1);
      else return(0);
   }

   //..........................................................................
   //
   // Description:  The dataSetToLocation converts a point from data grid
   //               coordinates to data coordinates.  This function returns
   //               the location of the point in grid coordinates; it should
   //               be re-implemented in any child class to provide a
   //               custom conversion between grid and data coordinates.
   //
   public String dataSetToLocation(
                                    Point pt   // in: point
                                  )
   {
      String loc;             // string containing location (output)
      Point  grid = pt;       // location

      // Invert y coordinate
      grid.y = mNy - 1 - grid.y;

      // Get data value
      grid.x = (grid.x >= 0 ? grid.x : 0);
      grid.x = (grid.x < mNx ? grid.x : (mNx-1));
      grid.y = (grid.y >= 0 ? grid.y : 0);
      grid.y = (grid.y < mNy ? grid.y : (mNy-1));

      loc = grid.x+" "+grid.y;

      return(loc);
   }
   //..........................................................................
   //
   // Description:  The dataSetWrite method writes the data to a file in
   //               a the chosen format.  This function is not implemented
   //               in the base class.
   //
   public void dataSetWrite(
                              String fileName // in: output file name
                            )
   {
   }
   //..........................................................................
   //
   // Description: The paintComponent method is called by the Java
   //              infrastructure when the image needs to be displayed.
   //
   public void paintComponent(Graphics g)
   {
      // Draw image on screen
      super.paintComponent(g);
      g.drawImage(mImage,0,0,mNx,mNy,this);
   }
   //..........................................................................
   //
   // Description:  The getPreferredSize method is called by the Java
   //               infrastructure when the preferred size of the image
   //               is required
   //
   public Dimension getPreferredSize()
   {
      // Return preferred image size
      Insets insets = getInsets();
      return new Dimension(mNx+insets.right+insets.left,mNy+insets.top+insets.bottom);
   }
   //..........................................................................
   // Data Members
   //
   protected int mN                    = 0;                    // header length in bytes/4 (int)
   protected int mType                 = 0;                    // image or data
   protected int mKind                 = 0;                    // integer or real
   protected int mSize                 = 0;                    // data size (1, 2, 4, or 8 bytes)
   protected int mNx                   = 0;                    // image width
   protected int mNy                   = 0;                    // image height
   protected int mOrgX                 = 0;                    // image x-origin
   protected int mOrgY                 = 0;                    // image y-origin
   protected int mResX                 = 0;                    // image x-resolution
   protected int mResY                 = 0;                    // image y-resolution
   protected double        mDataMin    = Double.MAX_VALUE;     // minimum data value
   protected double        mDataMax    = Double.MIN_VALUE;     // maximum data value
   protected double        mDc         = 0;                    // contour spacing
   protected double        mConMin     = 0;                    // min contour value
   protected int        [] mHead       = null;                 // header data
   protected Point         mLocMin     = null;                 // location of min
   protected Point         mLocMax     = null;                 // location of max
   protected byte     [][] mByteData   = null;                 // array for byte data storage
   protected short    [][] mShortData  = null;                 // array for short data storage
   protected int      [][] mIntData    = null;                 // array for int data storage
   protected float    [][] mFloatData  = null;                 // array for float data storage
   protected double   [][] mDoubleData = null;                 // array for double data storage
   protected int        [] mPixels     = null;                 // data in pixels
   protected Image         mImage      = null;                 // data image
//..............................................................................
// Static Members
//
   protected static int [] mRed        = {0,0,8,8,8,3,6,8,8};  // red components of color
   protected static int [] mGreen      = {0,6,7,6,7,8,8,8,8};  // green components of color
   protected static int [] mBlue       = {0,8,6,2,4,3,5,3,8};  // blue components of color
   protected static int    mNumCon     = 8;                    // number of contours
   protected static int    mINTENSITY  = 31;                   // max intensity
   protected static byte   mIMAGE      = 0;                    // data type (Image)
   protected static byte   mDATA       = 2;                    // data type (Data)
   protected static byte   mINT        = 0;                    // data kind (Integer)
   protected static byte   mREAL       = 2;                    // data kind (Real)
}
//..............................................................................
//  * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................
