//..............................................................................
// -*- Mode: C++; -*-
//                          Package   : DataHandler
// DataHandler.h            Author    : Deanna Evans (dke),
//                                      Ed Kraemer (eck) and,
//                                      Mark Pumar (map)
//
//  This file is part of DataHandler.
//
//  DataServer is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
// Description: The DataHandler functions are used to read, render, and supply 
//              data to the server. This class constitutes the configurable 
//              portion of the server.  Its public methods are called by the
//              JAVA wrapper and server to pass data to the browser.
//              This implementation reads a terrain elevation data file
//              in the proprietary FTD format.
//
//    THIS IS PART OF THE CONFIGURABLE INTERFACE AND HAS BEEN WRITTEN AS A
//    USGS SPECIFIC IMPLEMENTATION.
//..............................................................................
//
#ifndef _DATA_HEADER_H_
#define _DATA_HEADER_H_

// Include files
#include <stdio.h>
#include <string>

// Platform specific defines
#ifdef __WIN32__
#define DLL_EXPORT __declspec(dllexport)
#else
#define DLL_EXPORT
#endif

extern "C"
{
   DLL_EXPORT unsigned char * readFileAsImage(const char *fileName, int res, int &length);
   DLL_EXPORT unsigned char * readFileAsData (const char *fileName, int res, int &length);
   DLL_EXPORT int getFileInfo(const char *fileName,int &width, int &height, int &length);
}
// Enumerations
enum dataType {Byte    = 0, Short  = 2, Long = 4, Float = 8, Double = 16};
enum dataKind {isImage = 0, isData = 2};
enum dataFmt  {isInt   = 0, isReal = 2};

#endif

//..............................................................................
//*END * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................
