//..............................................................................
// -*- Mode: C++; -*-
//                          Package   : DataHandler
// OrbConfig.cpp            Author    : Deanna Evans (dke),
//                                      Ed Kraemer (eck) and,
//                                      Mark Pumar (map) 
//
//  This file is part of DataHandler.
//
//  DataHandler is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
// Description: The functions herein are used to configure the orb and naming
//              service.
//
//    THIS IS PART OF THE CONFIGURABLE INTERFACE AND HAS BEEN WRITTEN AS A
//    USGS SPECIFIC IMPLEMENTATION.
//..............................................................................
//
#include "OrbConfig.h"

// Global variables
const int MSG_SIZE = 3*1024*1024; // 3 MB, Maximum message size 

extern "C"
{
   //...........................................................................
   //
   // Description:  Get Required Message Size
   //
   DLL_EXPORT unsigned long getMsgSize(void)
   {
      // Return the maximum message size defined
      return(MSG_SIZE); 
   }

   //...........................................................................
   //
   // Description:  Get Context Kind
   //
   DLL_EXPORT std::string getContextKind(void)
   {
      // Return the context kind for the CORBA program
      std::string kind("Context");
      return(kind);
   }

   //...........................................................................
   //
   // Description:  Get Context ID
   //
   DLL_EXPORT std::string getContextID(void)
   {
      // Return the context ID for the CORBA program
      std::string id("DataReader");
      return(id);
   }

   //...........................................................................
   //
   // Description:  Get Object Kind
   //
   DLL_EXPORT std::string getObjectKind(void)
   {
      // Return the object kind for the CORBA program
      std::string kind("Object");
       return(kind);
   }

   //...........................................................................
   //
   // Description:  Get Object ID
   //
   DLL_EXPORT std::string getObjectID(void)
   {
      // Return the object ID for the CORBA program
      std::string id("Data");
      return(id);
   }

   //...........................................................................
   //
   // Description:  Get File Extension
   //
   DLL_EXPORT std::string getFileExt(void)
   {
      // Return the file extension for the data files to be read
      std::string ext("*.usg");
      return(ext);
   }

   //...........................................................................
   //
   // Description:  Get Path Delimiter (System Dependent)
   //
   DLL_EXPORT std::string getPathDelimiter(void)
   {
      // Return the file path delimiter based on the platform type
#ifdef __WIN32__
      std::string pd("\\");
#else
      std::string pd("/");
#endif
      return(pd);
   }
} // extern C
//..............................................................................
//*END * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................
