//..............................................................................
// -*- Mode: C++; -*-
//                          Package   : DataServer
// DataInterface.h          Author    : Deanna Evans (dke),
//                                      Ed Kraemer (eck) and,
//                                      Mark Pumar (map) 
//
//  This file is part of DataServer.
//
//  DataServer is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
// Description: The DataInterface class provides access to functions for reading
//              the data contained in the shared library or dll.
//..............................................................................
//
#ifndef _DATA_INTERFACE_H_
#define _DATA_INTERFACE_H_

#include <string>

// System dependent include files and variables
#ifdef __WIN32__
#include <windows.h>
#else
#include "DataHandler.h"
#include "OrbConfig.h"
typedef unsigned int HINSTANCE;
#endif

class DataInterface
{
   public:
      // Constructors and destructors
                      DataInterface(void);
                     ~DataInterface(void);

      // Public interface
      unsigned char * diGetData(const char *fileName, int res, const char * type, 
                                int &length);
      int             diGetFileInfo(const char *fileName,int &width, int &height, int &length);
      unsigned long   diGetMsgSize(void);
      std::string     diGetContextKind(void);
      std::string     diGetContextID(void);
      std::string     diGetObjectKind(void);
      std::string     diGetObjectID(void);
      std::string     diGetFileExt(void);
      std::string     diGetPathDelimiter(void);
   private:
      // Private interface
      unsigned char * diReadFileAsData(const char *fileName,  int res, int &length);
      unsigned char * diReadFileAsImage(const char *fileName, int res, int &length);
      HINSTANCE       mLib;
};

#endif
//..............................................................................
//*END * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................
