//..............................................................................
// -*- Mode: C++; -*-
//                          Package   : DataServer
// DataReader.h             Author    : Deanna Evans (dke),
//                                      Ed Kraemer (eck) and,
//                                      Mark Pumar (map) 
//
//  This file is part of DataServer.
//
//  DataServer is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
// Description: The DataReader class contains the functions called by the
//              JAVA client.
//..............................................................................
//
#ifndef _DATA_READER_H_
#define _DATA_READER_H_

// Include files
#include "DataInterface.h"
#include "Data.h"
#include <string>

using _CORBA_MODULE Data;
using _CORBA_MODULE POA_Data;


class DataReader : public POA_Data::DataInfo,
	               public PortableServer::RefCountServantBase
{
   private:
      // Private data members
      std::string         mTheDir;
      std::string         mTheFiles;
      std::string         mTheRes;
      std::string         mPathDel;
      DataInterface*      mDataInterfacePtr;
   public:
      // Constructors and destructors
                          DataReader(void);
      virtual            ~DataReader(void);

      // Virtual public interface
      virtual CORBA::Long getData(const char *name, CORBA::Long res,
                                  const char *type, UnboundedData_out vals);
      virtual CORBA::Long getLocation(CORBA::String_out dirLoc);
      virtual CORBA::Long getPathDelimiter(CORBA::String_out dirLoc);
      virtual CORBA::Long getFiles(const char *dir,  CORBA::String_out fileList);
      virtual CORBA::Long getFileResolution(const char *file, CORBA::Long &width,
                                            CORBA::Long& height, CORBA::Long& length);
      virtual void        drSetServerMessageSize(void);

      // Public interface 
      std::string         drGetContextKind(void);
      std::string         drGetContextID(void);
      std::string         drGetObjectKind(void);
      std::string         drGetObjectID(void);
};
#endif
//..............................................................................
//*END * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................

