//..............................................................................
// -*- Mode: C++; -*-
//                          Package   : DataServer
// FileFind.h               Author    : Deanna Evans (dke),
//                                      Ed Kraemer (eck) and,
//                                      Mark Pumar (map) 
//
//  This file is part of DataServer.
//
//  DataServer is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
// Description: The functions define herein are used to return the path and 
//              list of  files in a directory. 
//              Adapted from the Microsoft Foundation Classes C++ library.
//..............................................................................
//
#ifndef _FILE_FIND_H_
#define _FILE_FIND_H_

// Include files
#include <string>

class FileFind
{
   public:
      // Constructors and destructors
                    FileFind(std::string dirName);
      virtual      ~FileFind(void);
        
      // Public interface
      unsigned long ffGetLength(void)   const;
      std::string   ffGetFileName(void) const;
      int           ffIsDots(void)      const;
      int           ffIsDirectory(void) const;
      int           ffHasExtension(std::string fileExt) const;
      void          ffClose(void);
      int           ffFindFile(std::string dirName);
      int           ffFindTheNextFile(void);

   protected:
      // Protected data members
      void*         m_pFoundInfo;
      void*         m_hContext;
      std::string   mDirName;
      char*         mStartDir;
};
#endif
//..............................................................................
//*END * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................


