//..............................................................................
// -*- Mode: C++; -*-
//                          Package   : DataServer
// FileService.cpp          Author    : Deanna Evans (dke),
//                                      Ed Kraemer (eck) and,
//                                      Mark Pumar (map)
//
//  This file is part of DataServer.
//
//  DataServer is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
// Description: The functions define herein are used to return the path and 
//              list of available data files. 
//..............................................................................
//
// Include files
#include "FileService.h"
#include "FileFind.h"
#include <iostream.h>

// Platform specific information
#ifdef __WIN32__
#include <direct.h>
#define PATH_DEL ".\\"
#else
#include <unistd.h>
#define PATH_DEL "./"
#endif

//..............................................................................
//
// Description:  Method to get the current working directory of the server
//
std::string FileService::fsGetAbsDir(void)
{
   std::string dirList = "";  // full directory
   char       *buffer  = 0;   // directory buffer

   // Get the current working directory of the server
   buffer = getcwd(buffer, 256);
   if(buffer == 0) return(dirList);
   else dirList = std::string(buffer);
   // Free the buffer memory
   free(buffer);
   return(dirList);
}

//..............................................................................
//
// Description:  Method to get a list of available data files
//
std::string FileService::fsGetFileList(
                                       std::string dirName, // in: directory name
                                       std::string fileExt  // in: file extension
                                      )
{
   std::string fileList("");   // string for the list of files

   FileFind   finder(dirName);

   // Check for subdirectories and files
   if(finder.ffFindFile("*.*"))
   { 
      // There are subdirectories or files in this directory
      do
      {
         // Determine if this is a valid subdirectory
         if(finder.ffIsDirectory() && !finder.ffIsDots())
	  {
            // This is a subdirectory, add the name to the string
            fileList += PATH_DEL;
            fileList += finder.ffGetFileName();
            fileList += '\t';
	  }  
      } while(finder.ffFindTheNextFile());

      // Close the current directory
      finder.ffClose();
   }

   // Find the data files with the input extension
   if (finder.ffFindFile(fileExt.c_str()))
   {
      do
      {
         // Determine if this file has the correct extension
         if(finder.ffHasExtension(fileExt.c_str()))
         {
           // This is a data file, add the name to the string 
           fileList += finder.ffGetFileName();
           fileList += '\t';
         }
      } while(finder.ffFindTheNextFile());

      // Close the current directory
      finder.ffClose();
   }

   return(fileList);
}
//..............................................................................
//*END * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................
